<?php
/**
 * Perfect Checkout - Admin.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Admin
 * @since   1.0.0
 * @version 1.0.9
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_Admin' ) ) {

	class Perfect_Checkout_Admin {

		/**
		 * Constructor
		 *
		 * @access  public
		 * @since   1.0.0
		 * @version 1.0.2
		 */
		public function __construct() {
			// Runs when the plugin is activated to check the environment.
			register_activation_hook( PERFECTCHECKOUT_FILE, array( $this, 'check_environment' ) );

			// Reverts WooCommerce pages removed or hidden upon deactivation of plugin.
			register_deactivation_hook( PERFECTCHECKOUT_FILE, array( $this, 'revert_wc_pages' ) );

			$this->includes();

			// Hides the option "Hide shipping costs until an address is entered".
			add_filter( 'woocommerce_shipping_settings', array( $this, 'hide_shipping_option' ), 0 );

			// Add a post display state for "PerfectCheckout" page.
			add_filter( 'display_post_states', array( $this, 'add_display_post_states' ), 11, 2 );
		} // END __construct()

		/**
		 * Checks the environment for the plugin.
		 *
		 * @access public
		 * @since  1.1.2
		 */
		public function check_environment() {
			/**
			* Deactivate either CoCart Lite and CoCart Pro to prevent conflict
			* if either of them are a version lower than the packaged versions.
			*/
			if ( Perfect_Checkout_Helpers::is_cocart_installed() ) {
				if ( Perfect_Checkout_Helpers::is_cocart_version_lte( '3.0.8' ) ) {
					deactivate_plugins( plugin_basename( COCART_FILE ) );
				}
			}

			if ( Perfect_Checkout_Helpers::is_cocart_pro_installed() ) {
				deactivate_plugins( plugin_basename( COCART_PRO_FILE ) );
			}

			// If plugin was already setup before then setup plugin.
			self::maybe_reactivated_plugin();
		} // END check_environment()

		/**
		 * This runs assuming that we maybe activating the plugin again.
		 * If plugin has already installed before then we simply setup
		 * Perfect Checkout page and settings again.
		 *
		 * @access  public
		 * @since   1.0.2
		 * @version 1.0.9
		 */
		public function maybe_reactivated_plugin() {
			if ( ! empty( get_site_option( 'perfectcheckout_version' ) ) ) {
				// Records the page ID's of the current set WooCommerce cart and checkout pages.
				Perfect_Checkout_Install::backup_wc_pages();

				// Set default settings.
				Perfect_Checkout_Install::set_default_settings();

				// Create new page for "PerfectCheckout" if one does not yet exist.
				Perfect_Checkout_Install::maybe_create_page();

				// Set "PerfectCheckout" as the new cart page.
				Perfect_Checkout_Install::maybe_update_page();
			}
		} // END reactivate_plugin()

		/**
		 * Set the checkout pages back to what they were supposed to be
		 * and remove our checkout page.
		 *
		 * @access  public
		 * @since   1.0.2
		 * @version 1.0.9
		 */
		public function revert_wc_pages() {
			// Remove Perfect Checkout page.
			self::maybe_delete_pc_page();

			$cart_page_id     = get_option( 'perfectcheckout_wc_cart_page_id' );
			$checkout_page_id = get_option( 'perfectcheckout_wc_checkout_page_id' );

			// Set the original cart page back.
			if ( $cart_page_id > 0 ) {
				$cart_page = get_post( $cart_page_id, 'ARRAY_A' );

				if ( 'draft' === $cart_page['post_status'] ) {
					$cart_page['post_status'] = 'publish';
					wp_update_post( $cart_page );
				}
			}

			// Set the original checkout page back.
			if ( $checkout_page_id > 0 ) {
				$checkout_page = get_post( $checkout_page_id, 'ARRAY_A' );

				if ( 'draft' === $checkout_page['post_status'] ) {
					$checkout_page['post_status'] = 'publish';
					wp_update_post( $checkout_page );
				}
			}

			// Set WooCommerce cart and checkout pages back.
			update_option( 'woocommerce_cart_page_id', $cart_page_id );
			update_option( 'woocommerce_checkout_page_id', $checkout_page_id );

			// Delete backup data.
			delete_option( 'perfectcheckout_wc_cart_page_id' );
			delete_option( 'perfectcheckout_wc_checkout_page_id' );
		} // END revert_wc_pages()

		/**
		 * Delete "Perfect Checkout" page.
		 *
		 * @access  protected
		 * @since   1.0.2
		 * @version 1.0.9
		 * @return  void
		 */
		protected function maybe_delete_pc_page() {
			$pc_page_id = get_option( 'perfectcheckout_page_id' );

			if ( ! empty( $pc_page_id ) ) {
				wp_delete_post( $pc_page_id, true );
			}

			delete_option( 'perfectcheckout_page_id' );
		} // END maybe_delete_pc_page()

		/**
		 * Include any classes we need within admin.
		 *
		 * @access public
		 */
		public function includes() {
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-action-links.php';  // Action Links.
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-assets.php';        // Admin Assets.
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-plugin-screen.php'; // Admin Plugin Screen.
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-settings.php';      // Admin Settings.
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin-menus.php';         // Admin Menus.
		} // END includes()

		/**
		 * Hides the option "Hide shipping costs until an address is entered".
		 *
		 * @access public
		 * @param  array $settings - Original settings.
		 * @return array $settings - New settings
		 */
		public function hide_shipping_option( $settings ) {
			unset( $settings[2] );

			return $settings;
		} // END hide_shipping_option()

		/**
		 * Add a post display state for "Perfect Checkout" page in the page list table.
		 *
		 * @access  public
		 * @since   1.0.2
		 * @version 1.0.9
		 * @param   array   $post_states An array of post display states.
		 * @param   WP_Post $post        The current post object.
		 */
		public function add_display_post_states( $post_states, $post ) {
			if ( get_option( 'perfectcheckout_page_id' ) === $post->ID ) {
				if ( wc_get_page_id( 'checkout' ) !== get_option( 'perfectcheckout_page_id' ) ) {
					$post_states['wc_page_for_cart'] = __( 'Perfect Checkout Page', 'perfect-checkout' );
				} else {
					$post_states['wc_page_for_checkout'] = __( 'Perfect Checkout Page', 'perfect-checkout' );
				}
			}

			if ( get_option( 'perfectcheckout_wc_cart_page_id' ) === $post->ID ) {
				if ( wc_get_page_id( 'cart' ) !== get_option( 'perfectcheckout_wc_cart_page_id' ) ) {
					$post_states['wc_cart_page'] = __( 'WooCommerce Cart Page', 'perfect-checkout' );
				}
			}

			if ( get_option( 'perfectcheckout_wc_checkout_page_id' ) === $post->ID ) {
				if ( wc_get_page_id( 'checkout' ) !== get_option( 'perfectcheckout_wc_checkout_page_id' ) ) {
					$post_states['wc_page_for_checkout'] = __( 'WooCommerce Checkout Page', 'perfect-checkout' );
				}
			}

			return $post_states;
		} // END add_display_post_states()

	} // END class

} // END if class exists

return new Perfect_Checkout_Admin();
