<?php
/**
 * Perfect Checkout - Connect store controller.
 *
 * Handles the request to connect the store to Perfect Checkout.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\API
 * @since   1.3.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * REST API Connect store controller class.
 *
 * @package Perfect Checkout\API
 */
class Perfect_Checkout_API_Connect_Store_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'pc/connect';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = 'store';

	/**
	 * Register routes.
	 *
	 * @access public
	 */
	public function register_routes() {
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'connect_callback' ),
				'permission_callback' => array( $this, 'update_items_permissions_check' ),
				'args'                => array(
					'license' => array(
						'required'          => true,
						'description'       => __( 'Unique license key.', 'perfect-checkout' ),
						'type'              => 'string',
						'sanitize_callback' => 'sanitize_text_field',
						'validate_callback' => 'rest_validate_request_arg',
					),
				),
			)
		);
	} // register_routes()

	/**
	 * Need write/create permission.
	 *
	 * @access public
	 */
	public function update_items_permissions_check() {
		if ( ! wc_rest_check_user_permissions( 'create' ) ) {
			return new WP_Error( 'perfectcheckout_rest_cannot_create', __( 'Sorry, you are not allowed to create resources.', 'perfect-checkout' ), array( 'status' => rest_authorization_required_code() ) );
		}

		return true;
	} // END update_items_permissions_check()

	/**
	 * Returns settings of Perfect Checkout.
	 *
	 * @access public
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return WP_REST_Response
	 */
	public function connect_callback( $request ) {
		/**
		 * Get and sanitize license.
		 */
		$license = ! empty( $request['license'] ) ? sanitize_text_field( $request['license'] ) : false;

		/**
		 * Check a license was provided.
		*/
		if ( empty( $license ) ) {
			return new WP_Error( 'perfectcheckout_no_license_found', __( 'No license found. Please provide a license key.', 'perfect-checkout'), array('status' => 404 ) );
		}

		/**
		 * Update license.
		 */
		update_option( 'perfectcheckout_license', $license );

		/**
		 * Prepare response.
		 */
		$data = array(
			'updated' => true,
			'license' => get_option( 'perfectcheckout_license' ),
		);

		// Response.
		$response = rest_ensure_response( $data );
		$response->set_status( 200 );

		return $response;
	} // END connect_callback()

} // END class
