<?php
/**
 * Perfect Checkout - Installation related functions and actions.
 *
 * @author  Sébastien Dumont
 * @package Perfect Checkout\Install
 * @since   1.0.2
 * @version 1.0.9
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Perfect_Checkout_Install' ) ) {

	class Perfect_Checkout_Install {

		/**
		 * Constructor.
		 *
		 * @access public
		 */
		public function __construct() {
			// Checks version of Perfect Checkout and install/update if needed.
			add_action( 'init', array( $this, 'check_version' ), 5 );

			// Redirect to License page once activated.
			add_action( 'activated_plugin', array( $this, 'redirect_license' ), 10, 2 );
		} // END __construct()

		/**
		 * Check plugin version and run the updater if necessary.
		 *
		 * This check is done on all requests and runs if the versions do not match.
		 *
		 * @access public
		 * @static
		 */
		public static function check_version() {
			if ( ! defined( 'IFRAME_REQUEST' ) && version_compare( get_option( 'perfectcheckout_version' ), PerfectCheckout::$version, '<' ) && current_user_can( 'install_plugins' ) ) {
				self::install();
				do_action( 'perfectcheckout_updated' );
			}
		} // END check_version()

		/**
		 * Install Perfect Checkout.
		 *
		 * @access  public
		 * @static
		 * @since   1.0.2
		 * @version 1.0.9
		 */
		public static function install() {
			if ( ! is_blog_installed() ) {
				return;
			}

			// Check if we are not already running this routine.
			if ( 'yes' === get_transient( 'perfectcheckout_installing' ) ) {
				return;
			}

			// If we made it till here nothing is running yet, lets set the transient now for five minutes.
			set_transient( 'perfectcheckout_installing', 'yes', MINUTE_IN_SECONDS * 5 );
			if ( ! defined( 'PERFECTCHECKOUT_INSTALLING' ) ) {
				define( 'PERFECTCHECKOUT_INSTALLING', true );
			}

			// Records the page ID's of the current set WooCommerce cart and checkout pages.
			self::backup_wc_pages();

			// Set default settings.
			self::set_default_settings();

			// Create new page for "PerfectCheckout" if one does not yet exist.
			self::maybe_create_page();

			// Set "PerfectCheckout" as the new checkout page.
			self::maybe_update_page();

			// Set activation date.
			self::set_install_date();

			// Update plugin version.
			self::update_version();

			delete_transient( 'perfectcheckout_installing' );

			do_action( 'perfectcheckout_installed' );
		} // END install()

		/**
		 * Backup WooCommerce pages.
		 *
		 * @access public
		 * @static
		 * @since  1.0.9
		 */
		public static function backup_wc_pages() {
			$cart_page_id     = get_option( 'woocommerce_cart_page_id' );
			$checkout_page_id = get_option( 'woocommerce_checkout_page_id' );

			add_option( 'perfectcheckout_wc_cart_page_id', $cart_page_id );
			add_option( 'perfectcheckout_wc_checkout_page_id', $checkout_page_id );
		} // END backup_wc_pages()

		/**
		 * Sets default settings for PerfectCheckout.
		 *
		 * @access public
		 * @static
		 * @return void
		 */
		public static function set_default_settings() {
			// "Hide shipping costs until an address is entered" is disabled.
			update_option( 'woocommerce_shipping_cost_requires_address', 'no' );

			// Set checkout page to nothing.
			update_option( 'woocommerce_cart_page_id', '' );
			update_option( 'woocommerce_checkout_page_id', '' );
		} // set_default_settings()

		/**
		 * Create page on installation.
		 *
		 * @access public
		 * @static
		 */
		public static function maybe_create_page() {
			$page_id = wc_get_page_id( 'cart' );

			// If page does not already exist then create checkout page.
			if ( ! empty( $page_id ) || $page_id < 1 ) {
				self::create_page();
			}
		} // END maybe_create_page()

		/**
		 * Set "Perfect Checkout" as the new cart page if WooCommerce does not have it set already.
		 *
		 * @access  public
		 * @static
		 * @since   1.0.2
		 * @version 1.0.9
		 * @return  void
		 */
		public static function maybe_update_page() {
			$pc_page_id = get_option( 'perfectcheckout_page_id' );
			$wc_page_id = get_option( 'perfectcheckout_wc_cart_page_id' );

			if ( ! empty( $pc_page_id ) && $pc_page_id !== $wc_page_id ) {
				update_option( 'woocommerce_cart_page_id', $pc_page_id );
				log_me( 'New cart page set!' );
			}
		} // END maybe_update_page()

		/**
		 * Get checkout page details.
		 *
		 * @access  public
		 * @static
		 * @since   1.0.2
		 * @version 1.0.6
		 * @return  array $checkout
		 */
		public static function get_page_details() {
			$checkout = array(
				'name'    => _x( 'perfect-checkout', 'Page slug', 'perfect-checkout' ),
				'title'   => _x( 'Perfect Checkout', 'Page title', 'perfect-checkout' ),
				'content' => '',
				'parent'  => '',
			);

			return $checkout;
		} // END get_page_details()

		/**
		 * Create cart page that this plugin relies on.
		 *
		 * @access  public
		 * @static
		 * @since   1.0.2
		 * @version 1.0.9
		 * @param   string $slug
		 */
		public static function create_page( $slug = '' ) {
			include_once dirname( WC_PLUGIN_FILE ) . '/includes/admin/wc-admin-functions.php';

			$checkout = self::get_page_details();

			// If the slug is not empty then set it as the page slug.
			if ( ! empty( $slug ) ) {
				$checkout['name'] = $slug;
			}

			$pc_page_id = wc_create_page( esc_sql( $checkout['name'] ), 'woocommerce_cart_page_id', $checkout['title'], $checkout['content'], $checkout['parent'] );

			add_option( 'perfectcheckout_page_id', $pc_page_id );
			log_me( 'New PerfectCheckout page created.' );
		} // END create_page()

		/**
		 * Update plugin version to current.
		 *
		 * @access private
		 * @static
		 */
		private static function update_version() {
			update_option( 'perfectcheckout_version', PerfectCheckout::$version );
		} // END update_version()

		/**
		 * Set the time the plugin was installed.
		 *
		 * @access public
		 * @static
		 */
		public static function set_install_date() {
			add_option( 'perfectcheckout_install_date', time() );
		} // END set_install_date()

		/**
		 * Redirects to the License page upon plugin activation.
		 *
		 * @access public
		 * @static
		 * @param  string $plugin             Activate plugin file.
		 * @param  bool   $network_activation Whether to enable the plugin for all sites in the network
		 *                                    or just the current site. Multisite only.
		 */
		public static function redirect_license( $plugin, $network_activation ) {
			// Prevent redirect if plugin name does not match or multiple plugins are being activated.
			if ( plugin_basename( PERFECTCHECKOUT_FILE ) !== $plugin || isset( $_GET['activate-multi'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				return;
			}

			// If PerfectCheckout has already been installed before then don't redirect.
			if ( ! empty( get_option( 'perfectcheckout_version' ) ) && ! empty( get_option( 'perfectcheckout_install_date', time() ) ) ) {
				return;
			}

			$page = admin_url( 'admin.php' );

			$license = add_query_arg(
				array(
					'page' => 'perfect-checkout',
				),
				$page
			);

			/**
			 * Should Perfect Checkout be installed via WP-CLI,
			 * display a link to the License page.
			 */
			if ( defined( 'WP_CLI' ) && WP_CLI ) {
				WP_CLI::log(
					WP_CLI::colorize(
						'%y' . sprintf( '🎉 %1$s %2$s', __( 'Enter your license key for %3$s here:', 'perfect-checkout' ), $license, 'PerfectCheckout' ) . '%n'
					)
				);
				return;
			}

			wp_safe_redirect( $license );
			exit;
		} // END redirect_license()

	} // END class.

} // END if class exists.

return new Perfect_Checkout_Install();
