<?php
/**
 * Plugin Name:   Perfect Checkout
 * Plugin URI:    https://perfectcheckout.com
 * Description:   The absolute best checkout experience for WooCommerce.
 * Version:       1.3.0
 * Author:        Perfect Checkout
 * Author URI:    https://perfectcheckout.com
 * Text Domain:   perfect-checkout
 * Domain Path:   /languages/
 * Requires at least: 5.4
 * Requires PHP: 7.0
 * WC requires at least: 4.3
 * WC tested up to: 5.8
 * WP tested up to: 5.8
 *
 * @package PerfectCheckout
 */

defined( 'ABSPATH' ) || exit;

if ( ! defined( 'PERFECTCHECKOUT_FILE' ) ) {
	define( 'PERFECTCHECKOUT_FILE', __FILE__ );
}

if ( ! class_exists( 'PerfectCheckout' ) ) {
	class PerfectCheckout {

		/**
		 * Plugin Version
		 *
		 * @access public
		 * @static
		 * @var string $version - Version of plugin.
		 */
		public static $version = '1.3.0';

		/**
		 * Required WordPress Version
		 *
		 * @access public
		 * @static
		 * @var string $required_wp - Version of WordPress required.
		 */
		public static $required_wp = '5.4';

		/**
		 * Required WooCommerce Version
		 *
		 * @access public
		 * @static
		 * @var string $required_woo - Version of WooCommerce required.
		 */
		public static $required_woo = '4.3';

		/**
		 * Required PHP Version
		 *
		 * @access public
		 * @static
		 * @var string $required_php - Version of PHP required.
		 */
		public static $required_php = '7.0';

		/**
		 * @var object PerfectCheckout- the single instance of the class.
		 *
		 * @access protected
		 * @static
		 * @since 1.0.0
		 */
		protected static $_instance = null;

		/**
		 * The API server URL.
		 *
		 * @access protected
		 * @since 1.0.0
		 * @var string $API_URL
		 */
		protected $API_URL = 'https://my.perfectcheckout.com';

		/**
		 * Main Perfect Checkout Instance
		 *
		 * Ensures only one instance can be loaded
		 *
		 * @access  public
		 * @static
		 * @since   1.0.0
		 * @see     PerfectCheckout()
		 * @return  PerfectCheckout - Main instance
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		/**
		 * Load the plugin.
		 *
		 * @access  public
		 * @since   1.0.0
		 * @version 1.2.0
		 */
		public function __construct() {
			self::setup_constants();

			$pc_license           = get_option( 'perfectcheckout_license' );
			$shop_checkout_button = get_option( 'perfectcheckout_shop_checkout_button' );
			$checkout_button      = get_option( 'perfectcheckout_checkout_button' );

			include_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . '/includes/class-perfect-checkout-helpers.php';
			include_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . '/includes/class-perfect-checkout-install.php';
			include_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . '/includes/class-perfect-checkout-api.php';
			include_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . '/includes/pc-page-functions.php';

			if ( is_admin() ) {
				include_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . '/includes/admin/class-perfect-checkout-admin.php';
			}

			if ( ! empty( $pc_license ) ) {
				self::include_cocart();

				// Load CoCart package once WooCommerce is initialized.
				add_action( 'plugins_loaded', array( 'CoCart_Packages', 'init' ), 5 );
			}

			// Prepare CoCart.
			add_action( 'init', array( $this, 'lower_bridge' ), 0 );
			add_action( 'cocart_init', array( $this, 'prepare_cocart' ) );

			// Load Perfect Checkout connect gateway.
			add_action( 'plugins_loaded', array( $this, 'maybe_enable_gateway' ), 0 );
			add_action( 'plugins_loaded', array( $this, 'load_pc_gateway' ), 11 );

			// Remove 'checkout' from nav.
			add_filter( 'wp_nav_menu_objects', array( $this, 'exclude_menu_objects' ) );

			// Add CSRF headers.
			add_filter( 'wp_headers', array( $this, 'add_perfectcheckout_csrf_header' ) );

			// Adds buy now request ability.
			add_action( 'wp_loaded', array( $this, 'pc_buy_now' ), 15 );

			// Logout customer and migrate cart as new guest customer.
			add_action( 'wp_loaded', array( $this, 'logout_migration' ), 0 );

			// Login customer.
			add_action( 'wp_loaded', array( $this, 'login_customer' ), 0 );

			// load our custom page template for Perfect Checkout.
			add_filter( 'template_include', array( $this, 'perfectcheckout_page_template' ) );

			// Adds checkout button to product page if enabled.
			if ( ! empty( $pc_license ) && ! empty( $checkout_button ) && 'yes' === $checkout_button ) {
				add_action( 'woocommerce_simple_add_to_cart', array( $this, 'perfectcheckout_simple_add_to_cart' ), 32 );
			}

			// Adds checkout button to shop loop if enabled.
			if ( ! empty( $pc_license ) && ! empty( $shop_checkout_button ) && 'yes' === $shop_checkout_button ) {
				add_action( 'woocommerce_after_shop_loop_item', array( $this, 'perfectcheckout_shop_loop_add_to_cart' ), 11 );
			}

			// Prevents tax calculating on shop base when digital mode is enabled.
			add_filter( 'woocommerce_order_get_tax_location', array( $this, 'get_tax_location' ), 99, 2 );

			// Add Perfect Checkout connect gateway to WooCommerce.
			add_filter( 'woocommerce_payment_gateways', array( $this, 'add_perfect_checkout_gateway_connection' ) );

			// Hide Perfect Checkout connect gateway from original checkout.
			add_filter( 'woocommerce_available_payment_gateways', array( $this, 'hide_perfectcheckout_gateway' ) );

			// Preload the iframe.
			add_action( 'wp_head', array( $this, 'pc_preload' ) );

			// Remove the checkout button from the minicart.
			add_action(
				'woocommerce_widget_shopping_cart_buttons',
				function() {
					// Removing Buttons.
					remove_action( 'woocommerce_widget_shopping_cart_buttons', 'woocommerce_widget_shopping_cart_proceed_to_checkout', 20 );
				},
				1
			);

			// Load translation files.
			add_action( 'init', array( $this, 'load_plugin_textdomain' ), 0 );
		} // END __construct()

		/**
		 * Setup Constants
		 *
		 * @access public
		 * @static
		 * @since  1.2.0
		 */
		public static function setup_constants() {
			self::define( 'PERFECTCHECKOUT_VERSION', self::$version );
		} // END setup_constants()

		/**
		 * Define constant if not already set.
		 *
		 * @access private
		 * @static
		 * @since  1.2.0
		 * @param  string      $name Name of constant.
		 * @param  string|bool $value Value of constant.
		 */
		private static function define( $name, $value ) {
			if ( ! defined( $name ) ) {
				define( $name, $value );
			}
		} // END define()

		/**
		 * Includes CoCart Commercial
		 *
		 * @access public
		 * @return void
		 */
		public function include_cocart() {
			if ( ! Perfect_Checkout_Helpers::is_cocart_installed() || ! Perfect_Checkout_Helpers::is_cocart_pro_installed() ) {
				// Package loader.
				require_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . 'src/packages.php';

				// temporary until new packaged version is available.
				require plugin_dir_path( PERFECTCHECKOUT_FILE ) . 'packages/framework/load-package.php';
			}
		} // include_cocart()

		/**
		 * Prepare CoCart.
		 *
		 * @access  public
		 * @since   1.0.9
		 * @version 1.1.2
		 */
		public function prepare_cocart() {
			// Set CoCart Session Handler if it did not identiy it in time.
			if ( class_exists( 'WC_Session' ) && ! class_exists( 'CoCart_Session_Handler' ) ) {
				add_filter( 'woocommerce_session_handler', function( $session_handler ) {
					if ( Perfect_Checkout_Helpers::package_controller_exists( 'rest-api' ) ) {
						include_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . 'packages/rest-api/includes/class-cocart-session-handler.php';

						return 'CoCart_Session_Handler';
					}

					return $session_handler;
				} );
			}

			// Enables all cross origin header requests.
			add_filter( 'cocart_disable_all_cors', function() {
				return false;
			} );

			// Sets Cookie Samesite Attribute for CoCart if site is secure.
			add_filter( 'cocart_cookie_samesite', function( $samesite ) {
				if ( is_ssl() ) {
					return 'None; Secure';
				}

				return $samesite;
			} );
		} // END prepare_cocart()

		/**
		 * Disables security check for registered users in CoCart.
		 *
		 * @access public
		 * @since  1.1.2
		 */
		public function lower_bridge() {
			add_filter( 'cocart_secure_registered_users', function() {
				return false;
			} );
		} // END lower_bridge()

		/**
		 * Load Perfect Checkout connect gateway.
		 *
		 * @access public
		 * @since  1.3.0
		 */
		public function load_pc_gateway() {
			include_once plugin_dir_path( PERFECTCHECKOUT_FILE ) . '/includes/gateway/class-perfect-checkout-gateway.php';
		} // END load_pc_gateway()

		/**
		 * Enable "Perfect Checkout Connect Gateway".
		 *
		 * @access public
		 * @static
		 * @since  1.3.0
		 */
		public static function maybe_enable_gateway() {
			$tag = 'perfectcheckoutgateway';

			// If gateway has not been enabled then enable gateway.
			if ( ! get_transient( md5( $tag ) ) || ! get_option( 'woocommerce_perfect_settings' ) ) {
				$settings = array(
					'enabled' => 'yes'
				);

				update_option( 'woocommerce_perfect_settings', $settings );
				set_transient( md5( $tag ), true ); // Does not expire.
			}
		} // END maybe_enable_gateway()

		/**
		 * Remove WooCommerce 'Checkout' nav menu item.
		 *
		 * @todo Update to include Perfect Checkout nav menu item in its place.
		 *
		 * @access public
		 * @since  1.0.0
		 * @param  array $sorted_menu_objects - Nav menu before tweaks.
		 * @return array $sorted_menu_objects - Nav menu after tweaks.
		 */
		public function exclude_menu_objects( $sorted_menu_objects ) {
			$cart     = get_option( 'woocommerce_cart_page_id' );
			$checkout = get_option( 'woocommerce_checkout_page_id' );

			foreach ( $sorted_menu_objects as $key => $menu_object ) {
				if ( $menu_object->ID === $cart || $menu_object->ID === $checkout ) {
					unset( $items[ $key ] );
				}
			}

			return $sorted_menu_objects;
		} // END exclude_menu_objects()

		/**
		 * Gets the API url.
		 *
		 * Returns staging API URL if debug is enabled for Perfect Checkout.
		 *
		 * @access public
		 * @since  1.0.1
		 * @return string
		 */
		public function get_api_url() {
			if ( defined( 'PERFECTCHECKOUT_DEBUG' ) ) {
				return 'https://prelive.perfectcheckout.com';
			}

			return $this->API_URL;
		} // END get_api_url()

		/**
		 * Remove site header/footer/breadcrumb.
		 * Add the head to disable field zooming.
		 * Preload the necessary pages.
		 *
		 * @access public
		 */
		public function pc_preload() {
			$cart_key = strval( get_current_user_id() );
			$cookie   = WC()->session->get_session_cookie();
			if ( $cookie ) {
				$cart_key = $cookie[0];
			}
			$pc_license = get_option( 'perfectcheckout_license' );
			$frame_url  = esc_url( $this->get_api_url() . '?id=' . $pc_license . '&ck=' . $cart_key );
			?>
			<link rel="preconnect" href="https://fonts.googleapis.com" crossorigin>
			<link rel="preconnect" href="https://maps.googleapis.com" crossorigin>
			<link rel="prefetch" href='<?php echo esc_url( $frame_url ); ?>' as="document" crossorigin>
			<?php if ( is_page( array( 'perfect-checkout', 'Perfect Checkout' ) ) ) { ?>
			<style type="text/css">header.site-header { display: none; }</style>
			<style type="text/css">.site-content * { max-width: none !important; }</style>
			<style type="text/css">.storefront-breadcrumb { display: none;  }</style>
			<style type="text/css">footer.site-footer { display: none;  }</style>
			<style type="text/css">#wrapper-container { padding-top: 0 !important; padding-bottom: 0 !important; }</style>
			<style type="text/css">.topbar-device-mobile { display: none !important; }</style>
			<style type="text/css">@media (max-width: 568px) { .col-full { margin-left: 0 !important; margin-right: 0 !important; } #tbay-main-content { padding-bottom: 0 !important; } } </style>
			<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no" />
				<?php
			}
		} // END pc_preload()

		/**
		 * Includes the Perfect Checkout page template
		 * if detected that the user is on our checkout page.
		 *
		 * @access  public
		 * @since   1.0.0
		 * @version 1.0.5
		 * @param   string $page_template - Current page template if any.
		 * @return  string $page_template
		 */
		public function perfectcheckout_page_template( $page_template ) {
			if ( is_perfect_checkout() ) {
				$page_template = dirname( PERFECTCHECKOUT_FILE ) . '/templates/checkout.php';
			}

			return $page_template;
		} // END perfectcheckout_page_template()

		/**
		 * Add CSRF headers.
		 *
		 * @access public
		 * @since  1.0.0
		 * @param  array $headers - Before altering headers.
		 * @return array $headers - After altering headers.
		 */
		public function add_perfectcheckout_csrf_header( $headers ) {
			if ( ! empty( $_SERVER['HTTP_REFERER'] ) ) {
				$refer = sanitize_key( $_SERVER['HTTP_REFERER'] );

				$pos = strpos( $refer, (string) $this->get_api_url() );
				if ( false !== $pos ) {
					$headers['Access-Control-Allow-Origin'] = $this->get_api_url();

					return $headers;
				}
			}

			return $headers;
		} // END add_perfectcheckout_csrf_header()

		/**
		 * Adds a buy now request ability.
		 *
		 * Once items and coupons are added to cart the
		 * customer is redirected to perfect checkout page.
		 *
		 * @access public
		 * @since  1.1.4
		 */
		public function pc_buy_now() {
			// Make sure buy-now arg exists.
			if ( empty( $_REQUEST['pc-buy-now'] ) ) {
				return;
			}

			wc_nocache_headers();

			$product_ids = explode( ',', $_REQUEST['pc-buy-now'] );

			foreach ( $product_ids as $id_and_quantity ) {
				// Check for quantities defined in curie notation (<product_id>:<quantity>)
				$id_and_quantity = explode( ':', $id_and_quantity );
				$product_id      = absint( $id_and_quantity[0] );
				$quantity        = ! empty( $id_and_quantity[1] ) ? absint( $id_and_quantity[1] ) : 1;

				$was_added_to_cart = false;
				$adding_to_cart    = wc_get_product( $product_id );

				if ( ! $adding_to_cart ) {
					continue;
				}

				$add_to_cart_handler = apply_filters( 'woocommerce_add_to_cart_handler', $adding_to_cart->get_type(), $adding_to_cart );

				if ( 'variable' === $add_to_cart_handler || 'variation' === $add_to_cart_handler ) {
					self::add_to_cart_handler_variation( $product_id, $quantity );
				} else {
					self::add_to_cart_handler_simple( $product_id, $quantity );
				}
			}

			// Add coupons.
			$coupons = isset( $_REQUEST['coupons'] ) ? explode( ',', $_REQUEST['coupons'] ) : '';

			if ( ! empty( $coupons ) ) {
				foreach ( $coupons as $coupon ) {
					WC()->cart->add_discount( wc_format_coupon_code( wp_unslash( $coupon ) ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
				}
			}

			// Set customers email billing address.
			WC()->customer->set_props(
				array(
					'billing_email' => isset( $_REQUEST['email'] ) ? $_REQUEST['email'] : null,
				)
			);

			wp_safe_redirect( pc_get_checkout_url() );
			exit;
		} // END pc_buy_now()

		/**
		 * Logs out customer but keeps the cart items in the cart.
		 *
		 * @access public
		 * @since  1.3.0
		 */
		public function logout_migration() {
			// Make sure logout arg exists.
			if ( empty( $_REQUEST['pc-logout'] ) ) {
				return;
			}

			wc_nocache_headers();

			// Get current cart contents.
			$cart_contents = WC()->session->get( 'cart' );

			// Destroy cart and logout user.
			WC()->session->delete_cart( WC()->session->get_customer_id() );
			WC()->session->destroy_cookie();
			wp_logout();

			// Create new session if cart is not empty.
			if ( ! empty( $cart_contents ) ) {
				// Save cart contents as new guest session.
				$cart_key = WC()->session->create_new_cart( '', $cart_contents );

				// Redirect loop so Perfect Checkout knows which cart to load.
				wp_safe_redirect( add_query_arg(
					array(
						'pc-logout' => true,
						'pc-key'    => $cart_key
					), pc_get_checkout_url()
				) );
				exit;
			}

			// Load saved cart and set session.
			if ( ! empty( $_REQUEST['pc-key'] ) ) {
				$cart_key = $_REQUEST['pc-key'];

				$cart = maybe_unserialize( WC()->session->get_cart( $cart_key ) );

				WC()->session->set( 'cart', $cart );
				WC()->session->set_cart_hash();
				WC()->session->set_customer_id( $cart_key );
				WC()->session->set_cart_expiration();
				WC()->session->set_customer_cart_cookie( true );
			}
		} // END logout_migration()

		/**
		 * Logs in the customer.
		 *
		 * @access public
		 * @since  1.3.0
		 */
		public function login_customer() {
			// Make sure login arg exists.
			if ( empty( $_REQUEST['pc-login'] ) ) {
				return;
			}

			/** Allow for cross-domain requests (from the front end). */
			send_origin_headers();

			header( 'X-Robots-Tag: noindex' );

			wc_nocache_headers();

			$login = $_REQUEST['pc-login'];

			// Do nothing if login failed.
			if ( 'failed' === $login ) {
				return;
			}

			$login = base64_decode( $login );
			$login = explode( ':', $login );

			$username = '';
			$password = '';

			if ( ! empty( $login[0] ) && ! empty( $login[1] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				// Fallback to check if the username and password was passed via URL.
				$username = trim( sanitize_user( $login[0] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				$password = trim( sanitize_text_field( $login[1] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended

				// Check if the username provided was an email address and get the username if true.
				if ( is_email( $login[0] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
					$user     = get_user_by( 'email', $login[0] ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
					$username = $user->user_login; // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				}
			}

			// Only authenticate if a username and password is available to check.
			if ( ! empty( $username ) && ! empty( $password ) ) {
				$user = wp_authenticate( $username, $password );

				// Set authentication cookie.
				wp_set_auth_cookie( $user->ID, true, is_ssl() );

				/**
				 * Fires after the user has successfully logged in.
				 *
				 * @param string  $user_login Username.
				 * @param WP_User $user       WP_User object of the logged-in user.
				 */
				do_action( 'wp_login', $user->user_login, $user );
			} else {
				// Failed login.
				wp_safe_redirect( add_query_arg(
					array(
						'pc-login' => 'failed',
						'reason'   => 'missing-credentials'
					), pc_get_checkout_url()
				) );
			}

			if ( is_wp_error( $user ) ) {
				wp_safe_redirect( add_query_arg(
					array(
						'pc-login' => 'failed',
						'reason'   => 'invalid'
					), pc_get_checkout_url()
				) );
			}

			// Successful login.
			wp_safe_redirect( pc_get_checkout_url() );
			exit;
		} // END login_customer()

		/**
		 * Handle adding simple products to the cart.
		 *
		 * @access private
		 * @since  1.1.4
		 * @param  int $product_id Product ID to add to the cart.
		 * @param  int $quantity Quantity of product to add to the cart.
		 * @return bool success or not
		 */
		private function add_to_cart_handler_simple( $product_id, $quantity = '' ) {
			$quantity          = empty( $quantity ) ? 1 : wc_stock_amount( wp_unslash( $quantity ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );

			if ( $passed_validation && false !== WC()->cart->add_to_cart( $product_id, $quantity ) ) {
				wc_add_to_cart_message( array( $product_id => $quantity ), true );
				return true;
			}

			return false;
		} // END add_to_cart_handler_simple()

		/**
		 * Handle adding variation products to the cart.
		 *
		 * Note: The attributes for the variation are looked up and applied without additional parameter.
		 *
		 * @access private
		 * @since  1.1.4
		 * @param  int $variation_id Variation ID to add to the cart.
		 * @param  int $quantity Quantity of product to add to the cart.
		 * @return bool success or not
		 */
		private function add_to_cart_handler_variation( $variation_id, $quantity = '' ) {
			$quantity   = empty( $quantity ) ? 1 : wc_stock_amount( wp_unslash( $quantity ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$product    = wc_get_product( $variation_id );
			$product_id = $product->get_parent_id(); // Parent ID.

			// This skips verifying the attributes selected. Variations with attributes that have "ANY" are skipped.
			$attributes = wc_get_product_variation_attributes( $variation_id );

			$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity, $variation_id, $attributes );

			if ( ! $passed_validation ) {
				return false;
			}

			// Prevent parent variable product from being added to cart.
			if ( empty( $variation_id ) && $product && $product->is_type( 'variable' ) ) {
				return false;
			}

			if ( false !== WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variations ) ) {
				wc_add_to_cart_message( array( $product_id => $quantity ), true );
				return true;
			}

			return false;
		} // END add_to_cart_handler_variation()

		/**
		 * Adds checkout button to single product page.
		 *
		 * @access public
		 * @since  1.2.0
		 */
		public function perfectcheckout_simple_add_to_cart() {
			include_once dirname( PERFECTCHECKOUT_FILE ) . '/templates/single-product/add-to-cart/simple.php';
		} // END perfectcheckout_simple_add_to_cart()

		/**
		 * Adds checkout button to shop archive loop.
		 *
		 * @access public
		 * @since  1.2.0
		 */
		public function perfectcheckout_shop_loop_add_to_cart() {
			include dirname( PERFECTCHECKOUT_FILE ) . '/templates/loop/checkout.php';
		} // END perfectcheckout_shop_loop_add_to_cart()

		/**
		 * Prevents shop base being used to calculate tax when digital mode is enabled.
		 *
		 * @access public
		 * @since  1.2.0
		 * @param  array                                           $args - Address used to calculate tax.
		 * @param  object WC_Abstract_Order - Abstract Order class.
		 * @return array
		 */
		public function get_tax_location( $args, $object ) {
			// If no country for shipping or billing is set, then reset location.
			if ( ! $object->get_shipping_country() || ! $object->get_billing_country() ) {
				$args = array(
					'country'  => '',
					'state'    => '',
					'postcode' => '',
					'city'     => '',
				);
			}

			return $args;
		} // END get_tax_location()

		/**
		 * Adds Perfect Checkout gateway to WooCommerce.
		 *
		 * @access public
		 * @since  1.3.0
		 * @param  array $methods - Before methods altered.
		 * @return array $methods - After methods altered.
		 */
		public function add_perfect_checkout_gateway_connection( $methods ) {
			$methods[] = 'Perfect_Checkout_Gateway_Connection'; 

			array_unshift( $methods); // Sets it to the front of the line if gateway ordering is not already set.

			return $methods;
		} // END add_perfect_checkout_gateway_connection()

		/**
		 * Remove Perfect Checkout gateway from available 
		 * methods if on original checkout page.
		 *
		 * @access public
		 * @param  array $gateways - Before gateways modified.
		 * @return array $gateways - After gateways modified.
		 */
		public function hide_perfectcheckout_gateway( $gateways ){
			// Dot remove gateway if we are accessing the WordPress dashboard or any account page.
			if (
				is_admin() ||
				is_account_page() ||
				is_add_payment_method_page()
			) {
				return $gateways;
			}

			unset( $gateways[ 'perfect' ] );

			return $gateways; 
		} // END hide_perfectcheckout_gateway()

		/**
		 * Load the plugin translations if any ready.
		 *
		 * Note: the first-loaded translation file overrides any following ones if the same translation is present.
		 *
		 * Locales found in:
		 *      - WP_LANG_DIR/perfect-checkout/perfect-checkout-LOCALE.mo
		 *      - WP_LANG_DIR/plugins/perfect-checkout-LOCALE.mo
		 *
		 * @access public
		 * @since  1.3.0
		 */
		public function load_plugin_textdomain() {
			if ( function_exists( 'determine_locale' ) ) {
				$locale = determine_locale();
			} else {
				$locale = is_admin() ? get_user_locale() : get_locale();
			}

			$locale = apply_filters( 'plugin_locale', $locale, 'perfect-checkout' );

			unload_textdomain( 'perfect-checkout' );
			load_textdomain( 'perfect-checkout', WP_LANG_DIR . '/perfect-checkout/perfect-checkout-' . $locale . '.mo' );
			load_plugin_textdomain( 'perfect-checkout', false, plugin_basename( dirname( PERFECTCHECKOUT_FILE ) ) . '/languages' );
		} // END load_plugin_textdomain()

	} // END CLASS

} // END if class exists

/**
 * For debugging
 */
function log_me( $message ) {
	if ( WP_DEBUG === true ) {
		if ( is_array( $message ) || is_object( $message ) ) {
			error_log( print_r( $message, true ) );
		} else {
			error_log( $message );
		}
	}
}

/**
 * Returns the main instance of Perfect Checkout.
 *
 * @since  1.0.0
 * @return PerfectCheckout
 */
function PerfectCheckout() {
	// Check if WooCommerce is active.
	if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
		die( 'Woocommerce is not installed and active.' );
	}
	return PerfectCheckout::instance();
}

PerfectCheckout();
